@@ -6,16 +6,22 @@
 #
 # Python DB-API 2.0 module for Firebird. 
 ##############################################################################
+DEFAULT_CHARSET='UTF8'
+
 isc_info_end = 1
 isc_info_truncated = 2
 isc_info_error = 3
+isc_info_data_not_ready = 4
+isc_info_length = 126
+isc_info_flag_end = 127
+
 isc_info_db_id = 4
 isc_info_reads = 5
 isc_info_writes = 6
 isc_info_fetches = 7
 isc_info_marks = 8
 isc_info_implementation = 11
-isc_info_isc_version = 12
+isc_info_version = 12
 isc_info_base_level = 13
 isc_info_page_size = 14
 isc_info_num_buffers = 15
@@ -77,6 +83,26 @@ isc_info_next_transaction = 107
 isc_info_db_provider = 108
 isc_info_active_transactions = 109
 
+isc_info_svc_svr_db_info = 50
+isc_info_svc_get_license = 51
+isc_info_svc_get_license_mask = 52
+isc_info_svc_get_config = 53
+isc_info_svc_version = 54
+isc_info_svc_server_version = 55
+isc_info_svc_implementation = 56
+isc_info_svc_capabilities = 57
+isc_info_svc_user_dbpath = 58
+isc_info_svc_get_env = 59
+isc_info_svc_get_env_lock = 60
+isc_info_svc_get_env_msg = 61
+isc_info_svc_line = 62
+isc_info_svc_to_eof = 63
+isc_info_svc_timeout = 64
+isc_info_svc_get_licensed_users = 65
+isc_info_svc_limbo_trans = 66
+isc_info_svc_running = 67
+isc_info_svc_get_users = 68
+
 SQL_TYPE_TEXT = 452
 SQL_TYPE_VARYING = 448
 SQL_TYPE_SHORT = 500
@@ -96,6 +122,7 @@ ISOLATION_LEVEL_READ_UNCOMMITTED = 0
 ISOLATION_LEVEL_READ_COMMITED = 1
 ISOLATION_LEVEL_REPEATABLE_READ = 2
 ISOLATION_LEVEL_SERIALIZABLE = 3
+ISOLATION_LEVEL_READ_COMMITED_READ_ONLY = 4
 
 isc_tpb_version1 = 1
 isc_tpb_version3 = 3
@@ -126,6 +153,9 @@ isc_tpb_lock_timeout = 21
 isc_spb_version1 = 1
 isc_spb_current_version = 2
 isc_spb_version = isc_spb_current_version
+isc_spb_res_buffers = 9
+isc_spb_res_page_size = 10
+isc_spb_res_length = 11
 isc_spb_user_name = 28              # isc_dpb_user_name
 isc_spb_sys_user_name = 19          # isc_dpb_sys_user_name
 isc_spb_sys_user_name_enc = 31      # isc_dpb_sys_user_name_enc
@@ -153,6 +183,19 @@ isc_spb_trc_id = 1
 isc_spb_trc_name = 2
 isc_spb_trc_cfg = 3
 
+# isc_info_svc_svr_db_info params
+isc_spb_num_att = 5
+isc_spb_num_db = 6
+
+# isc_info_svc_db_stats params
+isc_spb_sts_data_pages = 0x01
+isc_spb_sts_db_log =0x02
+isc_spb_sts_hdr_pages = 0x04
+isc_spb_sts_idx_pages = 0x08
+isc_spb_sts_sys_relations = 0x10
+isc_spb_sts_record_versions = 0x20
+isc_spb_sts_table = 0x40
+isc_spb_sts_nocreation = 0x80
 
 # Service Action Items
 isc_action_svc_backup = 1
