@@ -14,61 +14,61 @@ from firebirdsql.fbcore import *
 class Services(Connection):
 
     def backup_database(self, database_name, backup_filename, callback=None):
-        spb = bs([isc_action_svc_backup])
+        spb = bytes([isc_action_svc_backup])
         s = self.str_to_bytes(database_name)
-        spb += bs([isc_spb_dbname]) + int_to_bytes(len(s), 2) + s
+        spb += bytes([isc_spb_dbname]) + int_to_bytes(len(s), 2) + s
         s = self.str_to_bytes(backup_filename)
-        spb += bs([isc_spb_bkp_file]) + int_to_bytes(len(s), 2) + s
+        spb += bytes([isc_spb_bkp_file]) + int_to_bytes(len(s), 2) + s
         if callback:
-            spb += bs([isc_spb_verbose])
+            spb += bytes([isc_spb_verbose])
         self._op_service_start(spb)
         (h, oid, buf) = self._op_response()
         self.svc_handle = h
         while True:
-            self._op_service_info(bs([0x02]), bs([0x3e]))
+            self._op_service_info(bytes([0x02]), bytes([0x3e]))
             (h, oid, buf) = self._op_response()
-            if buf[:4] == bs([0x3e,0x00,0x00,0x01]):
+            if buf[:4] == bytes([0x3e,0x00,0x00,0x01]):
                 break
             if callback:
                 ln = bytes_to_int(buf[1:3])
                 callback(self.bytes_to_str(buf[3:3+ln]))
 
     def restore_database(self, restore_filename, database_name, callback=None):
-        spb = bs([isc_action_svc_restore])
+        spb = bytes([isc_action_svc_restore])
         s = self.str_to_bytes(restore_filename)
-        spb += bs([isc_spb_bkp_file]) + int_to_bytes(len(s), 2) + s
+        spb += bytes([isc_spb_bkp_file]) + int_to_bytes(len(s), 2) + s
         s = self.str_to_bytes(database_name)
-        spb += bs([isc_spb_dbname]) + int_to_bytes(len(s), 2) + s
+        spb += bytes([isc_spb_dbname]) + int_to_bytes(len(s), 2) + s
         if callback:
-            spb += bs([isc_spb_verbose])
-        spb += bs([isc_spb_res_buffers,0x00,0x08,0x00,0x00,isc_spb_res_page_size,0x00,0x10,0x00,0x00,isc_spb_options,0x00,0x30,0x00,0x00])
+            spb += bytes([isc_spb_verbose])
+        spb += bytes([isc_spb_res_buffers,0x00,0x08,0x00,0x00,isc_spb_res_page_size,0x00,0x10,0x00,0x00,isc_spb_options,0x00,0x30,0x00,0x00])
         self._op_service_start(spb)
         (h, oid, buf) = self._op_response()
         self.svc_handle = h
         while True:
-            self._op_service_info(bs([0x02]), bs([0x3e]))
+            self._op_service_info(bytes([0x02]), bytes([0x3e]))
             (h, oid, buf) = self._op_response()
-            if buf[:4] == bs([0x3e,0x00,0x00,0x01]):
+            if buf[:4] == bytes([0x3e,0x00,0x00,0x01]):
                 break
             if callback:
                 ln = bytes_to_int(buf[1:3])
                 callback(self.bytes_to_str(buf[3:3+ln]))
 
     def trace_start(self, name=None, cfg=None, callback=None):
-        spb = bs([isc_action_svc_trace_start])
+        spb = bytes([isc_action_svc_trace_start])
         if name:
             s = self.str_to_bytes(name)
-            spb += bs([isc_spb_trc_name]) + int_to_bytes(len(s), 2) + s
+            spb += bytes([isc_spb_trc_name]) + int_to_bytes(len(s), 2) + s
         if cfg:
             s = self.str_to_bytes(cfg)
-            spb += bs([isc_spb_trc_cfg]) + int_to_bytes(len(s), 2) + s
+            spb += bytes([isc_spb_trc_cfg]) + int_to_bytes(len(s), 2) + s
         self._op_service_start(spb)
         (h, oid, buf) = self._op_response()
         self.svc_handle = h
         while True:
-            self._op_service_info(bs([0x02]), bs([0x3e]))
+            self._op_service_info(bytes([0x02]), bytes([0x3e]))
             (h, oid, buf) = self._op_response()
-            if buf[:4] == bs([0x3e,0x00,0x00,0x01]):
+            if buf[:4] == bytes([0x3e,0x00,0x00,0x01]):
                 break
             ln = bytes_to_int(buf[1:2])
             if callback:
@@ -76,13 +76,13 @@ class Services(Connection):
 
     def trace_stop(self, id, callback=None):
         id = int(id)
-        spb = bs([isc_action_svc_trace_stop])
-        spb += bs([isc_spb_trc_id]) + int_to_bytes(id, 4)
+        spb = bytes([isc_action_svc_trace_stop])
+        spb += bytes([isc_spb_trc_id]) + int_to_bytes(id, 4)
         self._op_service_start(spb)
         (h, oid, buf) = self._op_response()
         self.svc_handle = h
 
-        self._op_service_info(bs([0x02]), bs([0x3e]))
+        self._op_service_info(bytes([0x02]), bytes([0x3e]))
         (h, oid, buf) = self._op_response()
         ln = bytes_to_int(buf[1:2])
         if callback:
@@ -90,13 +90,13 @@ class Services(Connection):
 
     def trace_suspend(self, id, callback=None):
         id = int(id)
-        spb = bs([isc_action_svc_trace_suspend])
-        spb += bs([isc_spb_trc_id]) + int_to_bytes(id, 4)
+        spb = bytes([isc_action_svc_trace_suspend])
+        spb += bytes([isc_spb_trc_id]) + int_to_bytes(id, 4)
         self._op_service_start(spb)
         (h, oid, buf) = self._op_response()
         self.svc_handle = h
 
-        self._op_service_info(bs([0x02]), bs([0x3e]))
+        self._op_service_info(bytes([0x02]), bytes([0x3e]))
         (h, oid, buf) = self._op_response()
         ln = bytes_to_int(buf[1:2])
         if callback:
@@ -104,63 +104,63 @@ class Services(Connection):
 
     def trace_resume(self, id, callback=None):
         id = int(id)
-        spb = bs([isc_action_svc_trace_resume])
-        spb += bs([isc_spb_trc_id]) + int_to_bytes(id, 4)
+        spb = bytes([isc_action_svc_trace_resume])
+        spb += bytes([isc_spb_trc_id]) + int_to_bytes(id, 4)
         self._op_service_start(spb)
         (h, oid, buf) = self._op_response()
         self.svc_handle = h
 
-        self._op_service_info(bs([0x02]), bs([0x3e]))
+        self._op_service_info(bytes([0x02]), bytes([0x3e]))
         (h, oid, buf) = self._op_response()
         ln = bytes_to_int(buf[1:2])
         if callback:
             callback(self.bytes_to_str(buf[3:3+ln]))
 
     def trace_list(self, callback=None):
-        spb = bs([isc_action_svc_trace_list])
+        spb = bytes([isc_action_svc_trace_list])
         self._op_service_start(spb)
         (h, oid, buf) = self._op_response()
         self.svc_handle = h
         while True:
-            self._op_service_info(bs([0x02]), bs([0x3e]))
+            self._op_service_info(bytes([0x02]), bytes([0x3e]))
             (h, oid, buf) = self._op_response()
-            if buf[:4] == bs([0x3e,0x00,0x00,0x01]):
+            if buf[:4] == bytes([0x3e,0x00,0x00,0x01]):
                 break
             ln = bytes_to_int(buf[1:2])
             if callback:
                 callback(self.bytes_to_str(buf[3:3+ln]))
 
     def _getIntegerVal(self, item_id):
-        self._op_service_info(bs([]), bs([item_id]))
+        self._op_service_info(bytes([]), bytes([item_id]))
         (h, oid, buf) = self._op_response()
-        assert bi(buf[0]) == item_id
-        return bi(buf[1])
+        assert ord(buf[0]) == item_id
+        return ord(buf[1])
 
     def _getStringVal(self, item_id):
-        self._op_service_info(bs([]), bs([item_id]))
+        self._op_service_info(bytes([]), bytes([item_id]))
         (h, oid, buf) = self._op_response()
-        assert bi(buf[0]) == item_id
+        assert ord(buf[0]) == item_id
         ln = bytes_to_int(buf[1:3])
         return self.bytes_to_str(buf[3:3+ln])
 
     def _getSvrDbInfo(self):
-        self._op_service_info(bs([]), bs([isc_info_svc_svr_db_info]))
+        self._op_service_info(bytes([]), bytes([isc_info_svc_svr_db_info]))
         (h, oid, buf) = self._op_response()
-        assert bi(buf[0]) == isc_info_svc_svr_db_info
+        assert ord(buf[0]) == isc_info_svc_svr_db_info
         db_names=[]
         i = 1
-        while i < len(buf) and bi(buf[i]) != isc_info_flag_end:
-            if bi(buf[i]) == isc_spb_num_att:
+        while i < len(buf) and ord(buf[i]) != isc_info_flag_end:
+            if ord(buf[i]) == isc_spb_num_att:
                 num_attach =  bytes_to_int(buf[i+1:i+5])
                 i += 5
-            elif bi(buf[i]) == isc_spb_num_db:
+            elif ord(buf[i]) == isc_spb_num_db:
                 num_db =  bytes_to_int(buf[7:11])
                 i += 5
-            elif bi(buf[i]) == isc_spb_dbname:
-                ln = bytes_to_int(buf[i:i+1])
-                db_name = self.bytes_to_str(buf[i+2:i+2+ln])
+            elif ord(buf[i]) == isc_spb_dbname:
+                ln = bytes_to_int(buf[i+1:i+3])
+                db_name = self.bytes_to_str(buf[i+3:i+3+ln])
                 db_names.append(db_name)
-                i += ln + 2
+                i += 3 + ln
 
         return (num_attach, db_names)
 
@@ -170,9 +170,9 @@ class Services(Connection):
         self.svc_handle = h
         logs = ''
         while True:
-            self._op_service_info(bs([]), bs([0x3e]))
+            self._op_service_info(bytes([]), bytes([0x3e]))
             (h, oid, buf) = self._op_response()
-            if buf[:4] == bs([0x3e,0x00,0x00,0x01]):
+            if buf[:4] == bytes([0x3e,0x00,0x00,0x01]):
                 break
             ln = bytes_to_int(buf[1:2])
             logs += self.bytes_to_str(buf[3:3+ln]) + '\n'
@@ -209,7 +209,7 @@ class Services(Connection):
         return self._getSvrDbInfo()[1]
 
     def getLog(self):
-        spb = bs([isc_action_svc_get_fb_log])
+        spb = bytes([isc_action_svc_get_fb_log])
         return self._getLogLines(spb)
 
     def getStatistics(self, dbname, showOnlyDatabaseLogPages=False,
@@ -229,10 +229,10 @@ class Services(Connection):
         if showSystemTablesAndIndexes:
             optionMask |= isc_spb_sts_sys_relations
 
-        spb = bs([isc_spb_res_length])
+        spb = bytes([isc_spb_res_length])
         s = self.str_to_bytes(dbname)
-        spb += bs([isc_spb_dbname]) + int_to_bytes(len(s), 2) + s
-        spb += bs([isc_spb_options]) + int_to_bytes(optionMask, 4)
+        spb += bytes([isc_spb_dbname]) + int_to_bytes(len(s), 2) + s
+        spb += bytes([isc_spb_options]) + int_to_bytes(optionMask, 4)
         return self._getLogLines(spb)
 
 
